@@ -57,7 +57,7 @@ import struct
 import sys
 import warnings
 
-from .TiffTags import TYPES, TagInfo
+from .TiffTags import TYPES
 
 
 __version__ = "1.3.5"
@@ -141,34 +141,56 @@ OPEN_INFO = {
     (MM, 0, (1,), 1, (1,), ()): ("1", "1;I"),
     (II, 0, (1,), 2, (1,), ()): ("1", "1;IR"),
     (MM, 0, (1,), 2, (1,), ()): ("1", "1;IR"),
+    (II, 1, (1,), 1, (1,), ()): ("1", "1"),
+    (MM, 1, (1,), 1, (1,), ()): ("1", "1"),
+    (II, 1, (1,), 2, (1,), ()): ("1", "1;R"),
+    (MM, 1, (1,), 2, (1,), ()): ("1", "1;R"),
+
+    (II, 0, (1,), 1, (2,), ()): ("L", "L;2I"),
+    (MM, 0, (1,), 1, (2,), ()): ("L", "L;2I"),
+    (II, 0, (1,), 2, (2,), ()): ("L", "L;2IR"),
+    (MM, 0, (1,), 2, (2,), ()): ("L", "L;2IR"),
+    (II, 1, (1,), 1, (2,), ()): ("L", "L;2"),
+    (MM, 1, (1,), 1, (2,), ()): ("L", "L;2"),
+    (II, 1, (1,), 2, (2,), ()): ("L", "L;2R"),
+    (MM, 1, (1,), 2, (2,), ()): ("L", "L;2R"),
+
+    (II, 0, (1,), 1, (4,), ()): ("L", "L;4I"),
+    (MM, 0, (1,), 1, (4,), ()): ("L", "L;4I"),
+    (II, 0, (1,), 2, (4,), ()): ("L", "L;4IR"),
+    (MM, 0, (1,), 2, (4,), ()): ("L", "L;4IR"),
+    (II, 1, (1,), 1, (4,), ()): ("L", "L;4"),
+    (MM, 1, (1,), 1, (4,), ()): ("L", "L;4"),
+    (II, 1, (1,), 2, (4,), ()): ("L", "L;4R"),
+    (MM, 1, (1,), 2, (4,), ()): ("L", "L;4R"),
+
     (II, 0, (1,), 1, (8,), ()): ("L", "L;I"),
     (MM, 0, (1,), 1, (8,), ()): ("L", "L;I"),
     (II, 0, (1,), 2, (8,), ()): ("L", "L;IR"),
     (MM, 0, (1,), 2, (8,), ()): ("L", "L;IR"),
-    (II, 0, (3,), 1, (32,), ()): ("F", "F;32F"),
-    (MM, 0, (3,), 1, (32,), ()): ("F", "F;32BF"),
-    (II, 1, (1,), 1, (1,), ()): ("1", "1"),
-    (MM, 1, (1,), 1, (1,), ()): ("1", "1"),
-    (II, 1, (1,), 1, (4,), ()): ("L", "L;4"),
-    # ?
-    (II, 1, (1,), 2, (1,), ()): ("1", "1;R"),
-    (MM, 1, (1,), 2, (1,), ()): ("1", "1;R"),
     (II, 1, (1,), 1, (8,), ()): ("L", "L"),
     (MM, 1, (1,), 1, (8,), ()): ("L", "L"),
-    (II, 1, (1,), 1, (8, 8), (2,)): ("LA", "LA"),
-    (MM, 1, (1,), 1, (8, 8), (2,)): ("LA", "LA"),
     (II, 1, (1,), 2, (8,), ()): ("L", "L;R"),
     (MM, 1, (1,), 2, (8,), ()): ("L", "L;R"),
+
     (II, 1, (1,), 1, (12,), ()): ("I;16", "I;12"),
+
     (II, 1, (1,), 1, (16,), ()): ("I;16", "I;16"),
     (MM, 1, (1,), 1, (16,), ()): ("I;16B", "I;16B"),
     (II, 1, (2,), 1, (16,), ()): ("I;16S", "I;16S"),
     (MM, 1, (2,), 1, (16,), ()): ("I;16BS", "I;16BS"),
+
+    (II, 0, (3,), 1, (32,), ()): ("F", "F;32F"),
+    (MM, 0, (3,), 1, (32,), ()): ("F", "F;32BF"),
     (II, 1, (1,), 1, (32,), ()): ("I", "I;32N"),
     (II, 1, (2,), 1, (32,), ()): ("I", "I;32S"),
     (MM, 1, (2,), 1, (32,), ()): ("I;32BS", "I;32BS"),
     (II, 1, (3,), 1, (32,), ()): ("F", "F;32F"),
     (MM, 1, (3,), 1, (32,), ()): ("F", "F;32BF"),
+
+    (II, 1, (1,), 1, (8, 8), (2,)): ("LA", "LA"),
+    (MM, 1, (1,), 1, (8, 8), (2,)): ("LA", "LA"),
+
     (II, 2, (1,), 1, (8, 8, 8), ()): ("RGB", "RGB"),
     (MM, 2, (1,), 1, (8, 8, 8), ()): ("RGB", "RGB"),
     (II, 2, (1,), 2, (8, 8, 8), ()): ("RGB", "RGB;R"),
@@ -183,6 +205,7 @@ OPEN_INFO = {
     (MM, 2, (1,), 1, (8, 8, 8, 8), (2,)): ("RGBA", "RGBA"),
     (II, 2, (1,), 1, (8, 8, 8, 8), (999,)): ("RGBA", "RGBA"),  # Corel Draw 10
     (MM, 2, (1,), 1, (8, 8, 8, 8), (999,)): ("RGBA", "RGBA"),  # Corel Draw 10
+
     (II, 3, (1,), 1, (1,), ()): ("P", "P;1"),
     (MM, 3, (1,), 1, (1,), ()): ("P", "P;1"),
     (II, 3, (1,), 2, (1,), ()): ("P", "P;1R"),
@@ -201,10 +224,13 @@ OPEN_INFO = {
     (MM, 3, (1,), 1, (8, 8), (2,)): ("PA", "PA"),
     (II, 3, (1,), 2, (8,), ()): ("P", "P;R"),
     (MM, 3, (1,), 2, (8,), ()): ("P", "P;R"),
+
     (II, 5, (1,), 1, (8, 8, 8, 8), ()): ("CMYK", "CMYK"),
     (MM, 5, (1,), 1, (8, 8, 8, 8), ()): ("CMYK", "CMYK"),
+
     (II, 6, (1,), 1, (8, 8, 8), ()): ("YCbCr", "YCbCr"),
     (MM, 6, (1,), 1, (8, 8, 8), ()): ("YCbCr", "YCbCr"),
+
     (II, 8, (1,), 1, (8, 8, 8), ()): ("LAB", "LAB"),
     (MM, 8, (1,), 1, (8, 8, 8), ()): ("LAB", "LAB"),
 }
@@ -227,6 +253,7 @@ def _limit_rational(val, max_val):
 _load_dispatch = {}
 _write_dispatch = {}
 
+
 class IFDRational(Rational):
     """ Implements a rational class where 0/0 is a legal value to match
     the in the wild use of exif rationals.
@@ -266,7 +293,6 @@ class IFDRational(Rational):
             self._val = float('nan')
             return
 
-
         elif denominator == 1:
             if sys.hexversion < 0x2070000 and type(value) == float:
                 # python 2.6 is different.
@@ -284,7 +310,6 @@ class IFDRational(Rational):
     def denominator(a):
         return a._denominator
 
-
     def limit_rational(self, max_denominator):
         """
 
@@ -304,12 +329,12 @@ class IFDRational(Rational):
     def __hash__(self):
         return self._val.__hash__()
 
-    def __eq__(self,other):
+    def __eq__(self, other):
         return self._val == other
 
     def _delegate(op):
         def delegate(self, *args):
-            return getattr(self._val,op)(*args)
+            return getattr(self._val, op)(*args)
         return delegate
 
     """ a = ['add','radd', 'sub', 'rsub','div', 'rdiv', 'mul', 'rmul',
@@ -350,7 +375,6 @@ class IFDRational(Rational):
     __round__ = _delegate('__round__')
 
 
-
 class ImageFileDirectory_v2(collections.MutableMapping):
     """This class represents a TIFF tag directory.  To speed things up, we
     don't decode tags unless they're asked for.
@@ -450,11 +474,10 @@ class ImageFileDirectory_v2(collections.MutableMapping):
     def as_dict(self):
         """Return a dictionary of the image's tags.
 
-        use `dict(ifd)` instead.
-
         .. deprecated:: 3.0.0
         """
-        # FIXME Deprecate: use dict(self)
+        warnings.warn("as_dict() is deprecated. " +
+                      "Please use dict(ifd) instead.", DeprecationWarning)
         return dict(self)
 
     def named(self):
@@ -521,8 +544,7 @@ class ImageFileDirectory_v2(collections.MutableMapping):
                             self.tagtype[tag] = 2
 
         if self.tagtype[tag] == 7 and bytes is not str:
-            values = [value.encode("ascii", 'replace') if isinstance(value, str) else value
-                      for value in values]
+            values = [value.encode("ascii", 'replace') if isinstance(value, str) else value]
 
         values = tuple(info.cvt_enum(value) for value in values)
 
@@ -581,8 +603,7 @@ class ImageFileDirectory_v2(collections.MutableMapping):
 
     @_register_loader(1, 1)  # Basic type, except for the legacy API.
     def load_byte(self, data, legacy_api=True):
-        return (data if legacy_api else
-                tuple(map(ord, data) if bytes is str else data))
+        return data
 
     @_register_writer(1)  # Basic type, except for the legacy API.
     def write_byte(self, data):
@@ -1096,18 +1117,19 @@ class TiffImageFile(ImageFile.ImageFile):
         if DEBUG:
             print("- size:", self.size)
 
-        format = self.tag_v2.get(SAMPLEFORMAT, (1,))
-        if len(format) > 1 and max(format) == min(format) == 1:
+        sampleFormat = self.tag_v2.get(SAMPLEFORMAT, (1,))
+        if (len(sampleFormat) > 1
+            and max(sampleFormat) == min(sampleFormat) == 1):
             # SAMPLEFORMAT is properly per band, so an RGB image will
             # be (1,1,1).  But, we don't support per band pixel types,
             # and anything more than one band is a uint8. So, just
             # take the first element. Revisit this if adding support
             # for more exotic images.
-            format = (1,)
+            sampleFormat = (1,)
 
         # mode: check photometric interpretation and bits per pixel
         key = (
-            self.tag_v2.prefix, photo, format, fillorder,
+            self.tag_v2.prefix, photo, sampleFormat, fillorder,
             self.tag_v2.get(BITSPERSAMPLE, (1,)),
             self.tag_v2.get(EXTRASAMPLES, ())
             )
@@ -1126,8 +1148,8 @@ class TiffImageFile(ImageFile.ImageFile):
 
         self.info["compression"] = self._compression
 
-        xres = self.tag_v2.get(X_RESOLUTION,1)
-        yres = self.tag_v2.get(Y_RESOLUTION,1)
+        xres = self.tag_v2.get(X_RESOLUTION, 1)
+        yres = self.tag_v2.get(Y_RESOLUTION, 1)
 
         if xres and yres:
             resunit = self.tag_v2.get(RESOLUTION_UNIT, 1)
@@ -1192,7 +1214,7 @@ class TiffImageFile(ImageFile.ImageFile):
                 # https://github.com/python-pillow/Pillow/issues/279
                 if fillorder == 2:
                     key = (
-                        self.tag_v2.prefix, photo, format, 1,
+                        self.tag_v2.prefix, photo, sampleFormat, 1,
                         self.tag_v2.get(BITSPERSAMPLE, (1,)),
                         self.tag_v2.get(EXTRASAMPLES, ())
                         )
@@ -1416,14 +1438,15 @@ def _save(im, fp, filename):
         if hasattr(im, 'tag'):
             legacy_ifd = im.tag.to_v2()
         for tag, value in itertools.chain(ifd.items(),
-                                    getattr(im, 'tag_v2', {}).items(),
-                                    legacy_ifd.items()):
+                                          getattr(im, 'tag_v2', {}).items(),
+                                          legacy_ifd.items()):
             # Libtiff can only process certain core items without adding
             # them to the custom dictionary. It will segfault if it attempts
             # to add a custom tag without the dictionary entry
             #
             # UNDONE --  add code for the custom dictionary
-            if tag not in TiffTags.LIBTIFF_CORE: continue
+            if tag not in TiffTags.LIBTIFF_CORE:
+                continue
             if tag not in atts and tag not in blocklist:
                 if isinstance(value, unicode if bytes is str else str):
                     atts[tag] = value.encode('ascii', 'replace') + b"\0"
